/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Daniel A. Reed (reed@cs.uiuc.edu)
 * Contributing Author: Keith A. Shields (shields@cs.uiuc.edu)
 *
 * Project Manager and Principal Investigator:
 *      Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613, DARPA Contract No. DABT63-91-K-0004, by a grant
 * from the Digital Equipment Corporation External Research Program,
 * and by a collaborative research agreement with the Intel Supercomputer
 * Systems Division.
 *
 */
/*
 * PolarPlotFU.h: A dynamic polar plot plot functional unit
 *
 *	$Header: /mnt/Pablo-guitar/Stable.2-94/Visual/Src/System/Includes/RCS/PolarPlotFU.h,v 1.16 1994/03/15 16:25:57 aydt Exp $
 */

#ifndef PolarPlotFU_h
#define PolarPlotFU_h

#include "DisplayFunctionalUnit.h"
#include "Value.h"

class InputPort;
class PolarWrapper;
class WidgetBase;

class PolarPlotFU : public DisplayFunctionalUnit {
/*
*
* A PolarPlotFU is a functional unit that implements a dynamic polar plot
*
*/

private:
	static const char *const MY_CLASS;      // My class name

        Boolean_ isConfigured;  // Indicates if configure has been run
        int      magnitude;     // Maximum magnitude displayed

	InputPort	*input;
	WidgetBase 	*dialog;
	PolarWrapper 	*polar;

	Value inputValue;      // Data values for entire current array

public:
        /* Method PolarPlotFU:          The PolarPlotFU constructor     */
        PolarPlotFU();

        /* Method ~PolarPlotFU:         The PolarPlotFU destructor      */
        ~PolarPlotFU();

        /* Method configure:            Configure the functional unit   */
        virtual void configure();

        /* Method copy:                 Copy the functional unit.       */
        virtual FunctionalUnit * copy();
	
	/* Method fuCallback:          Callback code for this FU       */
	virtual void fuCallback( int nearestXValue, int nearestYValue );

        /* Method init:                 Initialize the polar plot       */
        virtual void init();

        /* Method loadConfigFromFile:   Loads configuration from a file */
        virtual Boolean_ loadConfigFromFile( const CString& fileName );

        /* Method ready:                Returns TRUE_ if the polar plot
        *                               is currently executable         */
        virtual Boolean_ ready();

        /* Method run:                  Execute the polar plot          */
        virtual void run( Boolean_& errorFlag );

        /* Method saveConfigToFile:     Saves configuration to a file   */
        virtual Boolean_ saveConfigToFile( const CString& fileName ) const;

};
#endif PolarPlotFU_h
