/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Taed Nelson (nelson@cs.uiuc.edu)
 * Contributing Author: Ruth A. Aydt (aydt@cs.uiuc.edu)
 *
 * Project Manager and Principal Investigator:
 *	Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613, DARPA Contract No. DABT63-91-K-0004, by a grant
 * from the Digital Equipment Corporation External Research Program,
 * and by a collaborative research agreement with the Intel Supercomputer
 * Systems Division.
 *
 */
/*
* PipeWriter.C: Code for the Abstract Class for sending pipe data.
*/

#include "PipeWriter.h"
#include "StreamPipe.h"
	
PipeWriter::PipeWriter( StreamPipe *thePipe )
{
	pipe[0] = thePipe;
	activePipes = 1;
	startOffset = -1;
}

PipeWriter::~PipeWriter( )
{
	/* */
}

Boolean_		/* virtual */
PipeWriter::addPipe( StreamPipe *thePipe )
{
	Boolean_ rc;

	if ( activePipes == ( PIPES_SUPPORTED - 1 ) ) {
            error( "Module already has maximum number(%d) of pipes attached", 
						PIPES_SUPPORTED );
	    rc = FAILURE_;
	} else if ( ! thePipe->isOutputPipe() ) {
            error( "DataStream %s not open for output", 
		    thePipe->getObjectName() );
	    rc = FAILURE_;
	} else {
	    pipe[activePipes] = thePipe;
	    activePipes++;
	    _writeSddfHeader();
	    rc = SUCCESS_;
	}
	return rc;
}

void
PipeWriter::deletePipe( StreamPipe *thePipe )
{
	int i, j;
	for ( i = 0; i < activePipes; i++ ) {
	    if ( pipe[i] == thePipe ) {
		break;
	    }
	}

	for ( j=i+1; j< activePipes; j++ ) {
	    pipe[j-1] = pipe[j];
	}

	activePipes--;
}

off_t
PipeWriter::getFileSize()
{
	if ( activePipes == 0 ) {
	    error( "::getFileSize() - No pipes connected to writer\n" );
	    return -1;
	} else {
	    return( pipe[0]->getFileSize() );
	}
}

off_t
PipeWriter::getOffset()
{
	if ( activePipes == 0 ) {
	    error( "::getOffset() - No pipes connected to writer\n" );
	    return -1;
	} else {
	    return( pipe[0]->getByteOffset() );
	}
}

void
PipeWriter::rewind()
{
	for ( int p = 0; p < activePipes; p++ ) {
	    pipe[p]->truncateAfterByte( startOffset );
	}
}

void
PipeWriter::setOffset( off_t offset )
{
	for ( int p = 0; p < activePipes; p++ ) {
	    pipe[p]->setByteOffset( offset );
	}
}

