/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Daniel A. Reed (reed@cs.uiuc.edu)
 * Project Manager and Principal Investigator:
 *	Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613, DARPA Contract No. DABT63-91-K-0004, by a grant
 * from the Digital Equipment Corporation External Research Program,
 * and by a collaborative research agreement with the Intel Supercomputer
 * Systems Division.
 *
 */
/*
 *	$Header: /mnt/Pablo-guitar/Stable.2-94/Visual/Src/Widgets/xygraph/Demo/RCS/xygraphmono.c,v 1.6 1994/02/25 04:47:26 aydt Exp $
 */

#include <X11/Xatom.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Box.h>

#include "XYGraph.h"

#define SetArg(r, v)	XtSetArg(arglist[argi], r, v); argi++;

#define COLORNUM	1
Pixmap	pixels[COLORNUM];

int	*xvalues;
int	*yvalues;
int	*ptcnt;

static void Select();
static XtCallbackRec selectcallback[] = {
  {Select,	(caddr_t) NULL	},
  {NULL,	(caddr_t) NULL	},
};

static XrmOptionDescRec options[] = {
  {"-lines",	"*xygraph.xygraphCnt",	XrmoptionSepArg,	NULL},
  {"-xticks",	"*xygraph.xticks",	XrmoptionSepArg,	NULL},
  {"-yticks",	"*xygraph.yticks",	XrmoptionSepArg,	NULL},
  {"-grid",	"*xygraph.gridColor",	XrmoptionSepArg, 	NULL},
  {"-showticks","*xygraph.showTicks",	XrmoptionNoArg,         "TRUE"},
  {"-showrange","*xygraph.showRange",	XrmoptionNoArg,		"TRUE"},
  {"-connect",  "*xygraph.connect",	XrmoptionNoArg,		"TRUE"},
  {"-logx",	"*xygraph.logx",	XrmoptionNoArg,		"TRUE"},
  {"-logy",	"*xygraph.logy",	XrmoptionNoArg,		"TRUE"},
};

Syntax(call)
char *call;
{
  printf("Usage: %s\n", call);
  printf("\t-lines <number>\n");
  printf("\t-xticks <number>\n");
  printf("\t-yticks <number>\n");
  printf("\t-grid <color>\n");
  printf("\t-showticks\n");
  printf("\t-showrange\n");
  printf("\t-connect\n");
  printf("\t-logx\n");
  printf("\t-logy\n");
  exit(1);

} /* Syntax */

void main(argc, argv)
int argc;
char **argv;
{
  int		i, j, lines, pts, argi;
  Widget	top, xygraph;
  Arg		arglist[20];
  XEvent	event;

  top = XtInitialize("main", "XYGraph", options,
		     XtNumber(options), &argc, argv);
  if (argc != 1)
    Syntax(argv[0]);

  argi = 0;
  SetArg(XtNwidth, 300);
  SetArg(XtNheight, 300);
  SetArg(XtNselect, selectcallback);
  SetArg(XtNautoscale, FALSE);
  SetArg(XtNmaxy, 1000);
  SetArg(XtNmaxx, 40);
  xygraph = XtCreateManagedWidget("xygraph", xygraphWidgetClass,
				 top, arglist, argi);

  XtRealizeWidget(top);

  argi = 0;
  SetArg(XtNxygraphCnt, 0);
  XtGetValues(xygraph, arglist, argi);
  lines = (int) arglist[0].value;
  pts = 40;
  ptcnt = (int *) XtMalloc(2*lines*sizeof(int));
  xvalues = (int *) XtMalloc(lines*pts*sizeof(int));
  yvalues = (int *) XtMalloc(lines*pts*sizeof(int));

  genbw(XtDisplay(top), XtScreen(top), COLORNUM);
  XYGraphSetColors(xygraph, COLORNUM, pixels);
  for (i=0; i<lines; i++) {
    ptcnt[i] = pts;
    for (j=0; j<pts; j++) {
      xvalues[i*pts+j] = j+1;
      yvalues[i*pts+j] = getvalue(i,j);
    }
  }
    XYGraphSetValues(xygraph, xvalues, yvalues, ptcnt, lines);

  while(1) {
    while (XtPending()) {
      XtNextEvent(&event);
      XtDispatchEvent(&event);
    }
    sleep(2);
    for (i=0; i<lines; i++) {
       ptcnt[i] = pts;
      for (j=0; j<pts; j++) {
        xvalues[i*pts+j] = j+1;
	yvalues[i*pts+j] = getvalue(i,j);
      }
    }
    XYGraphSetValues(xygraph, xvalues, yvalues, ptcnt, lines);
  }

} /* main */

static void Select(w, client_data, call_data)
Widget	w;
caddr_t	client_data;
caddr_t call_data;
{
  int	i;

  CoordinateLoc *xy = (CoordinateLoc *) call_data;
  
  printf("Callback:  x = %d  y = %d\n", xy -> x, xy -> y);

} /* Select */
